# Copyright 2005 PyTom <pytom@bishoujo.us>
#
# Permission is hereby granted, free of charge, to any person
# obtaining a copy of this software and associated documentation files
# (the "Software"), to deal in the Software without restriction,
# including without limitation the rights to use, copy, modify, merge,
# publish, distribute, sublicense, and/or sell copies of the Software,
# and to permit persons to whom the Software is furnished to do so,
# subject to the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

import gtk
import gtk.gdk
from gazpacho.loader.loader import ObjectBuilder
import os.path
import sys

import alice

class MainWindow(object):

    def __init__(self, state):
        """
        @param doll: The name (in ascii) of the doll associated with
        this window.
        """

        # Load the glade file.
        ob = ObjectBuilder(os.path.join(os.path.dirname(sys.argv[0]), 'mainwindow.glade'))

        self.win = ob.get_widget('mainwindow')

        self.config_combo = ob.get_widget('config_combo')
        self.cat_combo = ob.get_widget('category_combo')
        self.icons = ob.get_widget('icons')

        self.hscale = ob.get_widget('hscale')
        self.hcrop = ob.get_widget('hcrop')
        self.hoffset = ob.get_widget('hoffset')
        self.vscale = ob.get_widget('vscale')
        self.vcrop = ob.get_widget('vcrop')
        self.voffset = ob.get_widget('voffset')

        self.win.resize(400, 600)

        # Add in some of the change handlers.
        self.hscale.connect("value-changed", self.change_hscale)
        self.hcrop.connect("value-changed", self.change_hcrop)
        self.hoffset.connect("value-changed", self.change_hoffset)
        self.vscale.connect("value-changed", self.change_vscale)
        self.vcrop.connect("value-changed", self.change_vcrop)
        self.voffset.connect("value-changed", self.change_voffset)

        self.ignore_size_changed = False

        # Setup the config combo box.
        cell = gtk.CellRendererText()
        self.config_combo.pack_start(cell, True)
        self.config_combo.add_attribute(cell, 'text', 0)

        # Setup the categories combo box.
        cell = gtk.CellRendererText()
        self.cat_combo.pack_start(cell, True)
        self.cat_combo.add_attribute(cell, 'text', 0)

        # Create a RenderWindow.
        self.renderwindow = alice.renderer.RenderWindow()

        self.init(state)

        # Connect up signals, and show the widget.
        ob.signal_autoconnect(self)
        self.win.show()

    def init(self, state):
        """
        Called to initialize us with a new state, after init, load, or new.
        """

        # The state object associated with this window.
        self.state = state
        
        # The doll object associated with this window.
        self.doll = alice.dolls.dolls[state.doll]

        # The configuration being currently shown to the user.
        self.config = None

        # A list of categories found in the doll.
        self.categories = list(self.doll.layers)
        self.categories.sort()

        # Setup the categories combo box.
        self.catstore = gtk.ListStore(str)

        # Setup the configs combo box.
        self.update_configs()

        for i in self.categories:
            self.catstore.append((i,))

        self.cat_combo.set_model(self.catstore)
        self.cat_combo.set_active(0)

        # Set up the model for the icons.
        self.update_layers()

        # Set up the sizes.
        self.init_sizes()

        self.render()
        self.dirty(False)

    def init_sizes(self):
        hscale = self.state.h.scale
        hcrop = self.state.h.crop
        hoffset = self.state.h.offset 

        self.hscale.set_value(hscale)
        self.hcrop.set_value(hcrop)
        self.hoffset.set_value(hoffset)

        vscale = self.state.v.scale
        vcrop = self.state.v.crop
        voffset = self.state.v.offset 

        self.vscale.set_value(vscale)
        self.vcrop.set_value(vcrop)
        self.voffset.set_value(voffset)

        

    def dirty(self, dirty=True):

        self.dirtied = dirty

        title = "ALICE - " + self.doll.name

        if self.state.filename:
            title += " - " + os.path.basename(self.state.filename)

        if dirty:
            title = "*" + title

        self.win.set_title(title)

    def load_thumbnail(self, path):
        """
        Loads a thumbnail from the given path.
        """

        pbl = gtk.gdk.PixbufLoader()
        pbl.write(path.read())
        pbl.close()
        return pbl.get_pixbuf()

    def update_configs(self):

        self.configs = list(self.state.configs)
        self.configs.sort()

        if self.config not in self.configs:
            self.config = self.configs[0]

        self.config_store = gtk.ListStore(str)

        for i in self.configs:
            self.config_store.append((i,))

        self.config_combo.set_model(self.config_store)

        self.config_combo.set_active(self.configs.index(self.config))

    def update_layers(self):

        catnum = self.cat_combo.get_active()
        if catnum < 0:
            return

        self.category = self.categories[catnum]

        self.layers = list(self.doll.layers[self.category])
        self.layers.sort()
        self.layer_store = gtk.ListStore(gtk.gdk.Pixbuf, str)

        for layer in self.layers:
            l = self.doll.layers[self.category][layer]
            pb = self.load_thumbnail(l.thumb)

            self.layer_store.append((pb, l.name))

        self.icons.set_model(self.layer_store)

        self.icons.unselect_all()

        config = self.state.configs[self.config]

        for i, layer in enumerate(self.layers):
            if (self.category, layer) in config:
                self.icons.select_path((i, ))

    def layers_changed(self, w):

        config = self.state.configs[self.config]
        config = [ i for i in config if i[0] != self.category ]

        for path in self.icons.get_selected_items():
            config.append((self.category, self.layers[path[0]]))

        self.state.configs[self.config] = config

        self.render()
        self.dirty(True)

    def category_changed(self, w):
        self.update_layers()

    def delete(self, w, e=None):
        if self.dirtied:
            if alice.interface.yesnobox(self.win, "There are unsaved changes. Are you sure you want to quit?"):
                return False
            else:
                return True

        return False

    def quit(self, e):
        if self.dirtied:
            if not alice.interface.yesnobox(self.win, "There are unsaved changes. Are you sure you want to quit?"):
                return

        self.win.destroy()

    def destroy(self, w):
        gtk.main_quit()

    def config_changed(self, w):
        self.config = self.configs[self.config_combo.get_active()]

        self.render()
        self.update_layers()

    def add_configuration(self, a):
        name = alice.interface.entrybox(self.win, "Add Configuration...", "Please enter the name of the new configuration.", self.config)

        if not name:
            return

        if name in self.state.configs:
            alice.interface.messagebox(self.win, "The configuration '%s' already exists, refusing to create it again." % name, True)
            return
        
        self.state.configs[name] = self.state.configs[self.config][:]
        self.config = name

        self.update_configs()
        self.update_layers()

        self.render()
        self.dirty()
        
    def rename_configuration(self, a):
        name = alice.interface.entrybox(self.win, "Rename Configuration...", "Please enter the new name for the configuration.", self.config)

        if not name:
            return

        if name == self.config:
            return

        if name in self.state.configs:
            alice.interface.messagebox(self.win, "The configuration '%s' already exists, refusing to rename to it." % name, True)
            return

        self.state.configs[name] = self.state.configs[self.config]
        del self.state.configs[self.config]

        self.config = name

        self.update_configs()
        self.update_layers()

        self.render()
        self.dirty()

    def delete_configuration(self, a):

        if len(self.state.configs) == 1:
            alice.interface.messagebox(self.win, "Deleting the last configuration is not allowed.", True)
            return

        if not alice.interface.yesnobox(self.win, "Are you sure you want to delete the configuration '%s'?" % self.config):
            return

        i = self.configs.index(self.config)
        i += 1
        if i >= len(self.configs):
            i = 0

        del self.state.configs[self.config]
        self.config = self.configs[i]

        self.update_configs()
        self.update_layers()

        self.render()
        self.dirty()

    def save(self, a):

        if not self.state.filename:
            self.save_as(a)
            return

        self.state.store()

        self.dirty(False)

    def save_as(self, a):
        fn = alice.interface.savebox(self.win, "Save as...", self.state.filename)

        if not fn:
            return

        self.state.filename = fn
        self.save(a)

    def open(self, a):
        if self.dirtied:
            if not alice.interface.yesnobox(self.win, "There are unsaved changes. Are you sure you want to open a project?"):
                return

        fn = alice.interface.loadbox(self.win, "Open...")

        if not fn:
            return

        s = state.load(fn)
        self.init(s)

    def new(self, a):
        if self.dirtied:
            if not alice.interface.yesnobox(self.win, "There are unsaved changes. Are you sure you want to create a new project?"):
                return

        choices = list(alice.dolls.dolls)
        choices.sort()

        doll = alice.interface.newbox(self.win, "New...", "Please choose the doll to use for the new project.", choices)

        if not doll:
            return

        s = alice.state.State(doll)
        self.init(s)

    def render(self):
        self.renderwindow.queue_render(self.state, self.config)

    def change_hscale(self, w):
        self.hcrop.set_range(1, self.hscale.get_value_as_int())
        self.hoffset.set_range(0,
                               self.hscale.get_value_as_int() -
                               self.hcrop.get_value_as_int())

        self.state.h.scale = self.hscale.get_value_as_int()
        self.render()
        self.dirty()

    def change_hcrop(self, w):
        self.hoffset.set_range(0,
                               self.hscale.get_value_as_int() -
                               self.hcrop.get_value_as_int())

        self.state.h.crop = self.hcrop.get_value_as_int()
        self.render()
        self.dirty()

    def change_hoffset(self, w):
        self.state.h.offset = self.hoffset.get_value_as_int()
        self.render()
        self.dirty()

    def change_vscale(self, w):
        self.vcrop.set_range(1, self.vscale.get_value_as_int())
        self.voffset.set_range(0,
                               self.vscale.get_value_as_int() -
                               self.vcrop.get_value_as_int())

        self.state.v.scale = self.vscale.get_value_as_int()
        self.render()
        self.dirty()

    def change_vcrop(self, w):
        self.voffset.set_range(0,
                               self.vscale.get_value_as_int() -
                               self.vcrop.get_value_as_int())

        self.state.v.crop = self.vcrop.get_value_as_int()
        self.render()
        self.dirty()

    def change_voffset(self, w):
        self.state.v.offset = self.voffset.get_value_as_int()
        self.render()
        self.dirty()

    def render_to_files(self, a):

        dir = alice.interface.dirbox(self.win, "Please choose a directory to render PNG files to.", default=self.state.renderdir)

        if dir is None:
            return

        self.state.renderdir = dir

        alice.renderer.render_all(self.state, dir)
        
        
    

